"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/plainstack.ts
var plainstack_exports = {};
__export(plainstack_exports, {
  defineConfig: () => defineConfig,
  defineHandler: () => defineHandler,
  defineInmemoryTask: () => defineInmemoryTask,
  defineJob: () => defineJob,
  defineMiddleware: () => defineMiddleware,
  defineTask: () => defineDatabaseTask,
  fileRouter: () => fileRouter,
  getApp: () => getApp,
  getDatabase: () => getDatabase,
  getLogger: () => getLogger,
  getWorker: () => getWorker,
  html: () => html,
  isolate: () => isolate,
  json: () => json,
  log: () => log,
  middleware: () => middleware,
  migrate: () => migrate,
  notFound: () => notFound,
  outbox: () => outbox,
  perform: () => perform,
  printRoutes: () => printRoutes,
  randomId: () => randomId,
  redirect: () => redirect,
  sendMail: () => sendMail,
  stream: () => stream,
  testHandler: () => testHandler,
  unstable_admin: () => unstable_admin
});
module.exports = __toCommonJS(plainstack_exports);

// src/config.ts
var import_node_path = __toESM(require("path"));
var defaultConfigPaths = {
  routes: "app/routes",
  cli: "app/cli",
  tasks: "app/tasks",
  forms: "app/forms",
  public: "public",
  migrations: "migrations",
  schema: "app/schema.ts"
};
var defaultConfigHttp = {
  port: 3e3,
  redirects: {}
};
var defaultConfigDatabase = {
  migrationsTable: "__drizzle_migrations"
};
function parseProcessEnvLogLevel() {
  const level = process.env.LOG_LEVEL;
  const allowedLevels = [
    "silly",
    "debug",
    "verbose",
    "http",
    "info",
    "warn",
    "error"
  ];
  if (level && allowedLevels.includes(level.toLocaleLowerCase())) {
    return level;
  }
  return void 0;
}
var defaultConfigLogger = {
  level: parseProcessEnvLogLevel() ?? "http",
  transports: [],
  logger: void 0
};
var expandedConfig;
function expandConfig(config2) {
  const paths = { ...config2.paths ?? {}, ...defaultConfigPaths };
  const absolutePaths = {
    routes: import_node_path.default.resolve(process.cwd(), paths.routes),
    cli: import_node_path.default.resolve(process.cwd(), paths.cli),
    tasks: import_node_path.default.resolve(process.cwd(), paths.tasks),
    forms: import_node_path.default.resolve(process.cwd(), paths.forms),
    public: import_node_path.default.resolve(process.cwd(), paths.public),
    migrations: import_node_path.default.resolve(process.cwd(), paths.migrations),
    schema: import_node_path.default.resolve(process.cwd(), paths.schema)
  };
  const expanded = {
    nodeEnv: config2.nodeEnv,
    paths: absolutePaths,
    http: { ...config2.http ?? {}, ...defaultConfigHttp },
    // TODO use deep merge
    logger: { ...config2.logger ?? {}, ...defaultConfigLogger },
    database: { ...config2.database ?? {}, ...defaultConfigDatabase },
    mail: config2.mail
  };
  expandedConfig = expanded;
  return expanded;
}
function defineConfig(config2) {
  return expandConfig(config2);
}

// src/log.ts
var import_winston = __toESM(require("winston"));
var LABEL_MAX_LENGTH = 12;
function getLabel(info) {
  let label = "";
  if (info.name && typeof info.name === "string") label = info.name;
  if (label.length > LABEL_MAX_LENGTH)
    return `${label.slice(0, LABEL_MAX_LENGTH - 2)}..`;
  return label;
}
var printFormat = import_winston.default.format.printf((info) => {
  return `[${info.timestamp}] ${getLabel(info)} ${info.level}: ${info.message}`;
});
var paddingFormat = import_winston.default.format((info) => {
  const label = getLabel(info);
  const paddingLevel = Math.max(LABEL_MAX_LENGTH - label.length, 0);
  const spacesLevel = " ".repeat(paddingLevel);
  info.level = `${spacesLevel}${info.level}`;
  return info;
})();
var consoleFormat = import_winston.default.format.combine(
  import_winston.default.format.colorize({ all: true }),
  import_winston.default.format.errors({ stack: true }),
  import_winston.default.format.timestamp({
    format: "hh:mm:ss.SSS A"
  }),
  import_winston.default.format.splat(),
  paddingFormat,
  printFormat
);
function getProdLogger(logger2, name) {
  return import_winston.default.createLogger({
    level: logger2.level,
    format: import_winston.default.format.combine(
      import_winston.default.format.errors({ stack: true }),
      import_winston.default.format.timestamp({
        format: "YYYY-MM-DD hh:mm:ss.SSS A"
      }),
      import_winston.default.format.json()
    ),
    transports: [
      new import_winston.default.transports.Console({
        format: consoleFormat
      }),
      ...logger2.transports
    ],
    defaultMeta: { name }
  });
}
function getDevLogger(logger2, name) {
  return import_winston.default.createLogger({
    level: logger2.level,
    format: consoleFormat,
    transports: [new import_winston.default.transports.Console()],
    defaultMeta: { name }
  });
}
function getLogger(name) {
  const { nodeEnv, logger: logger2 } = expandedConfig || {
    nodeEnv: "development",
    logger: defaultConfigLogger
  };
  if (logger2?.logger) return logger2.logger;
  return nodeEnv === "production" ? getProdLogger(logger2, name) : getDevLogger(logger2, name);
}
var log = getLogger("app");

// src/id.ts
var import_cuid2 = require("@paralleldrive/cuid2");
function randomId(prefix) {
  if (prefix) return `${prefix}_${(0, import_cuid2.createId)()}`;
  return (0, import_cuid2.createId)();
}

// src/file-router.ts
var import_promises = __toESM(require("fs/promises"));
var import_node_path2 = __toESM(require("path"));
var import_express = __toESM(require("express"));

// src/plain-response.ts
var import_html = require("@kitajs/html");
var import_suspense = require("@kitajs/html/suspense");
function isPlainResponse(r) {
  return r._tag === "PlainResponse";
}
async function sendPlainResponse(res, plainResponse) {
  if (res.headersSent) {
    console.warn("Headers already sent, cannot send response");
    return;
  }
  if (plainResponse.headers) {
    for (const [key, value] of Object.entries(plainResponse.headers)) {
      res.setHeader(key, value);
    }
  }
  if (plainResponse.status) {
    res.status(plainResponse.status);
  }
  if (plainResponse.body) {
    res.send(await plainResponse.body);
  } else if (plainResponse.htmlStream) {
    const htmlStream = (0, import_suspense.renderToStream)(plainResponse.htmlStream);
    htmlStream.pipe(res);
    htmlStream.on("end", () => {
      res.end();
    });
    htmlStream.on("error", (err) => {
      console.error("Error streaming response", err);
    });
  } else {
    console.error("No body or htmlStream provided");
    res.end();
  }
}
function html(html2, opts) {
  return {
    _tag: "PlainResponse",
    headers: {
      "Content-Type": "text/html; charset=utf-8"
    },
    body: html2,
    status: opts?.status
  };
}
function stream(htmlStream) {
  return {
    _tag: "PlainResponse",
    headers: {
      "Content-Type": "text/html; charset=utf-8",
      "Cache-Control": "no-transform"
    },
    htmlStream
  };
}
function redirect(path5, opts) {
  return {
    _tag: "PlainResponse",
    status: opts?.status ?? 302,
    headers: {
      Location: path5
    }
  };
}
function json(json3, opts) {
  return {
    _tag: "PlainResponse",
    headers: {
      "Content-Type": "application/json"
    },
    body: JSON.stringify(json3),
    status: opts?.status
  };
}
function notFound() {
  return {
    _tag: "PlainResponse",
    status: 404,
    body: "Not Found"
  };
}

// src/handler.ts
function defineHandler(handler) {
  return handler;
}
async function handleResponse(res, userResponse) {
  if (isPlainResponse(userResponse)) {
    await sendPlainResponse(res, userResponse);
  } else if (typeof userResponse === "string" || typeof userResponse === "object" && userResponse instanceof Promise) {
    await sendPlainResponse(res, html(userResponse));
    return;
  } else if (typeof userResponse === "object") {
    await sendPlainResponse(res, json(userResponse));
    return;
  } else if (typeof userResponse === "function") {
    userResponse();
    return;
  } else {
    throw new Error("Unexpected response type");
  }
}

// src/file-router.ts
var log2 = getLogger("router");
async function readRoutesFromFs(opts) {
  const { baseDir, currentDir } = opts;
  const routes = [];
  let files = [];
  try {
    files = await import_promises.default.readdir(currentDir || baseDir);
  } catch (e) {
    log2.error(`error reading directory: ${currentDir || baseDir}`);
    log2.error("see the error below");
    throw e;
  }
  for (const file of files) {
    const fullFilePath = import_node_path2.default.join(currentDir || baseDir, file);
    const stat = await import_promises.default.stat(fullFilePath);
    if (stat.isDirectory()) {
      log2.debug(`found directory: ${fullFilePath}`);
      const subRoutes = await readRoutesFromFs({
        baseDir,
        currentDir: fullFilePath
      });
      routes.push(...subRoutes);
    } else if (stat.isFile() && file.endsWith(".tsx")) {
      const relativePath = import_node_path2.default.relative(baseDir, fullFilePath);
      log2.debug(`discovered file route: ${relativePath}`);
      routes.push({ filePath: fullFilePath, routePath: relativePath });
    }
  }
  return routes;
}
async function loadFileRoutes(routes) {
  const loadedRoutes = [];
  for (const { filePath } of routes) {
    let loadedFileRoute;
    try {
      const module2 = await import(filePath);
      const handler = module2?.default ? module2.default : module2;
      if (handler?.GET) {
        if (typeof handler.GET !== "function") {
          throw new Error(`GET export in route ${filePath} is not a function`);
        }
        loadedFileRoute = { filePath, GET: handler.GET };
      }
      if (handler?.POST) {
        if (typeof handler.POST !== "function") {
          throw new Error(`POST export in route ${filePath} is not a function`);
        }
        if (loadedFileRoute) {
          loadedFileRoute.POST = handler.POST;
        } else {
          loadedFileRoute = { filePath, POST: handler.POST };
        }
      }
      if (!loadedFileRoute) {
        log2.error(`no exported GET or POST functions found in ${filePath}`);
      } else {
        loadedRoutes.push(loadedFileRoute);
      }
    } catch (e) {
      log2.error(e);
      throw new Error(
        `double check the route at ${filePath}. Make sure to export a GET or POST function.`
      );
    }
  }
  return loadedRoutes;
}
function getExpressRoutePath({
  dir,
  filePath
}) {
  if (filePath === `${dir}/index.tsx`) return "/";
  let relativeFilePath = filePath.replace(dir, "");
  if (!dir.startsWith("/")) {
    relativeFilePath = filePath.replace(`/${dir}`, "");
  }
  const expressPath = relativeFilePath.replace(/\/index.tsx$/, "").replace(/index.tsx$/, "").replace(/\.tsx$/, "").split("/").map((part) => {
    if (part.startsWith("[...") && part.endsWith("]")) {
      return `:${part.slice(4, -1)}(*)`;
    }
    if (part.startsWith("[") && part.endsWith("]")) {
      return `:${part.slice(1, -1)}`;
    }
    return part;
  }).join("/");
  log2.debug(`"${dir}": ${relativeFilePath} -> ${expressPath}`);
  return expressPath;
}
function expressRouter({
  loadedFileRoutes,
  dir
}) {
  const router = import_express.default.Router();
  for (const route of loadedFileRoutes) {
    const routePath = getExpressRoutePath({
      dir,
      filePath: route.filePath
    });
    if (route.GET) {
      router.get(routePath, async (req, res, next) => {
        try {
          const userResponse = await route.GET({
            req,
            res
          });
          await handleResponse(res, userResponse);
        } catch (e) {
          next(e);
        }
      });
    }
    if (route.POST) {
      router.post(routePath, async (req, res, next) => {
        try {
          const userResponse = await route.POST({
            req,
            res
          });
          await handleResponse(res, userResponse);
        } catch (e) {
          next(e);
        }
      });
    }
  }
  return router;
}
async function fileRouter(opts) {
  const dir = opts.dir;
  if (opts.loadedFileRoutes?.length) {
    return expressRouter({
      loadedFileRoutes: opts.loadedFileRoutes,
      dir
    });
  }
  if (opts.fileRoutes?.length) {
    const loadedFileRoutes2 = await loadFileRoutes(opts.fileRoutes);
    return expressRouter({
      loadedFileRoutes: loadedFileRoutes2,
      dir
    });
  }
  const fileRoutes = await readRoutesFromFs({
    baseDir: dir,
    ignorePatterns: opts.ignorePatterns
  });
  const loadedFileRoutes = await loadFileRoutes(fileRoutes);
  return expressRouter({ loadedFileRoutes, dir });
}

// src/print-routes.ts
var import_express_list_endpoints = __toESM(require("express-list-endpoints"));

// src/get-app.ts
var import_express2 = __toESM(require("express"));
async function getApp(config2) {
  const app = (0, import_express2.default)();
  await config2.http.middleware({ app, config: config2 });
  return app;
}

// src/print-routes.ts
async function printRoutes(config2) {
  const app = await getApp(config2);
  const endpoints = (0, import_express_list_endpoints.default)(app);
  const sorted = endpoints.sort((a, b) => {
    if (a.path < b.path) return -1;
    if (a.path > b.path) return 1;
    return 0;
  });
  console.log("Routes:");
  for (const endpoint of sorted) {
    for (const method of endpoint.methods) {
      console.log(method, endpoint.path);
    }
  }
}

// src/middleware.ts
var import_errorhandler = __toESM(require("errorhandler"));
var import_express3 = __toESM(require("express"));
var import_express_rate_limit = __toESM(require("express-rate-limit"));
var import_morgan = __toESM(require("morgan"));

// src/migrate.ts
var import_promises3 = __toESM(require("fs/promises"));
var import_node_path3 = __toESM(require("path"));
var import_drizzle_orm = require("drizzle-orm");
var import_migrator = require("drizzle-orm/better-sqlite3/migrator");

// src/get-database.ts
var import_better_sqlite3 = __toESM(require("better-sqlite3"));
var import_better_sqlite32 = require("drizzle-orm/better-sqlite3");
var import_logger = require("drizzle-orm/logger");
var log3 = getLogger("database");
var database;
var PlainwebLogWriter = class {
  write(message) {
    log3.debug(message);
  }
};
var logger = new import_logger.DefaultLogger({ writer: new PlainwebLogWriter() });
function getDatabase(config2) {
  if (database) return database;
  const dbUrl = config2.database.dbUrl;
  if (!dbUrl || dbUrl === "")
    throw new Error("no database URL found, make sure DB_URL is set");
  log3.info(`db url ${dbUrl}`);
  const connection = new import_better_sqlite3.default(dbUrl);
  for (const [key, value] of Object.entries(config2.database.pragma)) {
    connection.pragma(`${key} = ${value}`);
  }
  database = (0, import_better_sqlite32.drizzle)(connection, {
    schema: config2.database.schema,
    logger
  });
  return database;
}

// src/plainweb-fs.ts
var import_promises2 = __toESM(require("fs/promises"));
async function fileExists(filePath) {
  try {
    await import_promises2.default.access(filePath);
    return true;
  } catch {
    return false;
  }
}
async function directoryExists(dirPath) {
  try {
    await import_promises2.default.access(dirPath);
    return true;
  } catch {
    return false;
  }
}

// src/migrate.ts
var log4 = getLogger("migrate");
async function migrate(config2) {
  try {
    const migrationConfig = {
      migrationsFolder: config2.paths.migrations,
      migrationsTable: config2.database.migrationsTable,
      migrationsSchema: config2.paths.schema
    };
    log4.info("applying migrations...");
    log4.debug("using migration config", migrationConfig);
    const db = getDatabase(config2);
    (0, import_migrator.migrate)(db, migrationConfig);
    log4.info("migrations have been applied");
  } catch (error2) {
    log4.error("could not apply migrations, something went wrong");
    throw error2;
  }
}
async function getLatestLocalMigrationTimestamp(config2) {
  log4.debug("getting latest local migration timestamp");
  if (!await directoryExists(config2.paths.migrations)) {
    log4.debug("migrations directory does not exist yet");
    return void 0;
  }
  const journalPath = import_node_path3.default.join(
    config2.paths.migrations,
    "meta",
    "_journal.json"
  );
  log4.debug("looking for migrations in journal", journalPath);
  if (!await fileExists(journalPath)) {
    log4.warn("no _journal.json file found in migrations folder");
    return void 0;
  }
  const journal = await import_promises3.default.readFile(journalPath, "utf-8");
  log4.debug("journal content", journal);
  const { entries } = JSON.parse(journal);
  if (!entries || entries.length === 0) {
    log4.debug("journal found but it's empty");
    return;
  }
  const latestEntry = entries[entries.length - 1];
  if (!latestEntry.when || typeof latestEntry.when !== "number") {
    log4.warn(
      "No or invalid 'when' field found in latest migration entry, something seems to be wrong"
    );
    return void 0;
  }
  log4.debug("latest local migration timestamp", latestEntry.when);
  return latestEntry.when;
}
async function getLatestAppliedMigrationTimestamp(config2) {
  log4.debug("getting latest applied migration timestamp");
  const db = getDatabase(config2);
  try {
    const query = import_drizzle_orm.sql`SELECT * FROM ${import_drizzle_orm.sql.identifier(
      config2.database.migrationsTable
    )} ORDER BY created_at DESC LIMIT 1`;
    const found = db.all(query);
    if (!found[0]) {
      log4.debug("no migrations found in database");
      return void 0;
    }
    log4.debug("latest migration timestamp", found[0].created_at);
    return found[0].created_at;
  } catch (error2) {
    if (error2 instanceof Error && error2.message.includes("no such table")) {
      log4.debug("migrations table not found in database");
      return void 0;
    }
    throw error2;
  }
}
async function hasPendingMigrations(config2) {
  log4.debug("checking for pending migrations");
  const latestLocalTimestamp = await getLatestLocalMigrationTimestamp(config2);
  if (!latestLocalTimestamp) {
    log4.debug("no migrations found, no migrations pending");
    return false;
  }
  const latestAppliedTimestamp = await getLatestAppliedMigrationTimestamp(config2);
  if (!latestAppliedTimestamp) {
    log4.debug("no migrations found in database, migrations pending");
    return true;
  }
  if (latestLocalTimestamp > latestAppliedTimestamp) {
    log4.debug(
      "local migrations found that have not been applied yet, migrations pending"
    );
    return true;
  }
  return false;
}

// src/middleware.ts
var log5 = getLogger("middleware");
function preferHeader(request, from, to) {
  const preferredValue = request.get(from.toLowerCase());
  if (preferredValue == null) return;
  delete request.headers[to];
  request.headers[to.toLowerCase()] = preferredValue;
}
function flyHeaders() {
  return function flyHeaders2(req, res, next) {
    if (process.env.FLY_APP_NAME == null) return next();
    log5.debug("setting fly headers");
    req.app.set("trust proxy", true);
    preferHeader(req, "Fly-Client-IP", "X-Forwarded-For");
    preferHeader(req, "Fly-Forwarded-Port", "X-Forwarded-Port");
    preferHeader(req, "Fly-Forwarded-Proto", "X-Forwarded-Protocol");
    preferHeader(req, "Fly-Forwarded-Ssl", "X-Forwarded-Ssl");
    return next();
  };
}
function migrations(config2) {
  return async (req, res, next) => {
    if (config2.nodeEnv === "production") return next();
    log5.info("checking for pending migrations");
    const pendingMigrations = await hasPendingMigrations(config2);
    if (!pendingMigrations) return next();
    log5.info("pending migrations found");
    await migrate(config2);
    next();
  };
}
function forceWWW(config2) {
  return function forceWWW2(req, res, next) {
    if (config2.nodeEnv === "development") return next();
    const host = req.header("host");
    if (host) {
      const wwwRegex = /^www\./i;
      const isWww = wwwRegex.test(host);
      const isSubdomain = host.split(".").length > 2;
      if (!isWww && !isSubdomain) {
        const lowercaseHost = host.toLowerCase();
        const newUrl = `https://www.${lowercaseHost}${req.url}`;
        log5.info("redirecting to", newUrl);
        res.redirect(301, newUrl);
      } else {
        next();
      }
    } else {
      next();
    }
  };
}
function logging(config2) {
  const logger2 = getLogger();
  return (0, import_morgan.default)(
    ":method :url :status :res[content-length] - :response-time ms",
    {
      stream: {
        write: (message) => logger2.http(message.trim())
      }
    }
  );
}
function error(config2) {
  if (config2.nodeEnv === "development") return (0, import_errorhandler.default)();
  return (err, req, res, next) => {
    log5.error(err);
    next(err);
  };
}
function staticFiles({
  nodeEnv,
  dir
}) {
  if (nodeEnv === "development") return import_express3.default.static(dir);
  return (req, res, next) => next();
}
function rateLimit({
  nodeEnv,
  rateLimit: rateLimit2
}) {
  if (nodeEnv !== "development") return (req, res, next) => next();
  return (0, import_express_rate_limit.default)({
    windowMs: rateLimit2?.windowMs ?? 60 * 1e3,
    limit: rateLimit2?.limit ?? 60,
    message: rateLimit2?.message ?? "Too many requests, please try again in a few seconds"
  });
}
function redirect2({
  redirects
}) {
  return (req, res, next) => {
    const target = redirects[req.path];
    if (target) {
      log5.info("redirecting to", target);
      res.redirect(target);
    } else {
      next();
    }
  };
}
function json2() {
  return import_express3.default.json();
}
function urlencoded() {
  return import_express3.default.urlencoded({ extended: true });
}
function database2({
  database: database3
}) {
  return (req, res, next) => {
    log5.debug("attach database to request");
    res.locals.database = database3;
    next();
  };
}
async function fileRouter2({
  dir
}) {
  return fileRouter({ dir });
}
function id() {
  return (req, res, next) => {
    res.locals.id = randomId("req");
    next();
  };
}
var middleware = {
  flyHeaders,
  forceWWW,
  logging,
  error,
  redirect: redirect2,
  staticFiles,
  rateLimit,
  json: json2,
  urlencoded,
  database: database2,
  fileRouter: fileRouter2,
  migrations,
  id
};
function defineMiddleware(stack) {
  return stack;
}

// src/test-handler.ts
var import_node_mocks_http = require("node-mocks-http");
async function testHandler(handler, req, {
  database: database3
}) {
  const res = (0, import_node_mocks_http.createResponse)();
  res.locals.database = database3;
  const userResponse = await handler({ req, res });
  await handleResponse(res, userResponse);
  return res;
}

// src/mail.ts
var import_nodemailer = __toESM(require("nodemailer"));
var log6 = getLogger("mail");
var devTransporter = import_nodemailer.default.createTransport({
  jsonTransport: true
});
var outbox = [];
function instantiateMailer(config2) {
  if ("host" in config2 && "port" in config2 && "secure" in config2 && "auth" in config2) {
    return import_nodemailer.default.createTransport(config2);
  }
  return config2;
}
async function sendMail(config2, mailOptions) {
  if (process.env.NODE_ENV === "test" || process.env.NODE_ENV === "development") {
    const result = await devTransporter.sendMail(mailOptions);
    outbox.push(result);
    log6.info("email sent to dev outbox -------------", mailOptions);
    log6.info(result.envelope);
    log6.info(result.message);
    return;
  }
  await instantiateMailer(config2).sendMail(mailOptions);
}

// src/isolate.ts
var import_drizzle_orm2 = require("drizzle-orm");
async function isolate(db, fn) {
  if (process.env.NODE_ENV !== "test") {
    throw new Error("Make sure NODE_ENV=test is set when running tests");
  }
  let err = null;
  try {
    db.run(import_drizzle_orm2.sql.raw("BEGIN"));
    try {
      await fn(db);
    } catch (e) {
      err = e;
    }
  } finally {
    db.run(import_drizzle_orm2.sql.raw("ROLLBACK"));
  }
  if (err) {
    const e = new Error(`Rethrowing error: "${err.message}"`);
    e.original_error = err;
    e.stack = err.stack;
    throw e;
  }
}

// src/task/work-tasks.ts
var import_promises4 = __toESM(require("fs/promises"));
var import_node_path4 = __toESM(require("path"));
var import_node_util = __toESM(require("util"));

// src/task/task.ts
var log7 = getLogger("tasks");
function isTask(task) {
  return typeof task === "object" && task !== null && "perform" in task && "start" in task && typeof task.start === "function" && typeof task.perform === "function";
}

// src/task/work-tasks.ts
async function loadTasks(tasksDir) {
  log7.verbose(`loading tasks from ${tasksDir}`);
  if (!await directoryExists(tasksDir)) {
    log7.info("no tasks found to run");
    return [];
  }
  try {
    const files = await import_promises4.default.readdir(tasksDir);
    const taskFiles = files.filter((file) => import_node_path4.default.extname(file) === ".ts");
    if (files.length !== taskFiles.length) {
      log7.warn(
        "warning: found non-task files or folders in the tasks directory"
      );
    }
    return (await Promise.all(
      taskFiles.map(async (file) => {
        const filePath = import_node_path4.default.join(tasksDir, file);
        log7.debug(`loading task from ${filePath}`);
        let taskModule = await import(filePath);
        if (!taskModule.default) {
          log7.error(`no default export found in task at ${filePath}`);
          return;
        }
        if (taskModule.default?.default) {
          taskModule = taskModule.default;
        }
        if (!isTask(taskModule.default)) {
          log7.error(`default export in ${filePath} is not a valid task`);
          log7.error(import_node_util.default.inspect(taskModule.default, { depth: 3 }));
          return;
        }
        const task = taskModule.default;
        return task;
      })
    )).filter((t) => !!t);
  } catch (error2) {
    log7.error("error loading tasks:", error2);
    return [];
  }
}
function workLoadedTasks(loadedTasks) {
  if (loadedTasks.length > 0) {
    log7.info("starting tasks:");
    for (const task of loadedTasks) {
      log7.info(`   * ${task.name}`);
    }
  } else {
    log7.info("no tasks to run");
  }
  try {
    const timeouts = loadedTasks.map((task) => ({
      name: task.name,
      timeout: task.start()
    }));
    return timeouts.reduce(
      (record, item) => {
        record[item.name] = item.timeout;
        return record;
      },
      {}
    );
  } catch (error2) {
    log7.error("error running tasks:", error2);
    return {};
  }
}
async function workTasks(config2, error2) {
  const loadedTasks = await loadTasks(config2.paths.tasks);
  log7.debug(`loaded ${loadedTasks.length} tasks`);
  return workLoadedTasks(loadedTasks);
}
async function perform(task, data) {
  try {
    await task.perform(data ?? void 0);
  } catch (error2) {
    log7.error("error performing task:", error2);
  }
}

// src/get-worker.ts
function getWorker(config2) {
  return {
    /** Load all tasks and work them. This must be called before any tasks can be enqueued. */
    start: () => {
      log7.verbose("start task worker");
      return workTasks(config2);
    }
  };
}

// src/job.ts
function defineJob(opts) {
  return {
    name: opts.name,
    process: opts.process,
    success: opts.success,
    failure: opts.failure
  };
}

// src/task/database-task.ts
var import_drizzle_orm3 = require("drizzle-orm");
var import_drizzle_orm4 = require("drizzle-orm");
var import_drizzle_orm5 = require("drizzle-orm");
var import_drizzle_orm6 = require("drizzle-orm");
var import_sqlite_core = require("drizzle-orm/sqlite-core");

// src/task/define-task.ts
function sanitizeTaskName(name) {
  const fileName = name.split("/").pop() || name;
  const nameWithoutExtension = fileName.split(".").slice(0, -1).join(".");
  const sanitizedName = nameWithoutExtension.replace(/\s+/g, "-");
  return sanitizedName;
}
function defineTaskWithStorageAdapter({
  enqueue,
  fetch,
  success: adapterSuccess,
  failure: adapterFailure
}, {
  name,
  process: process2,
  success,
  failure,
  pollIntervall = 1e3,
  batchSize = 1,
  maxRetries = 5,
  retryIntervall = 1e3 * 3
}) {
  log7.verbose(`defining task "${name}"`);
  return {
    name: sanitizeTaskName(name),
    perform: async (data) => {
      await enqueue({ data, name });
    },
    start() {
      if (pollIntervall < 0)
        throw new Error(`pollIntervall in task ${name} must be positive`);
      log7.verbose(`starting task: ${name}`);
      let executing = false;
      const timeout = setInterval(async () => {
        if (executing) {
          log7.info(`protect task ${name} from overrun`);
          return;
        }
        executing = true;
        const start = Date.now();
        try {
          log7.verbose(`Run task ${name} for ${pollIntervall}ms`);
          while (Date.now() - start < pollIntervall) {
            try {
              const tasks2 = await fetch({
                name,
                batchSize,
                maxRetries,
                retryIntervall
              });
              if (!tasks2 || Array.isArray(tasks2) && tasks2.length === 0) {
                break;
              }
              log7.verbose(
                `fetched ${Array.isArray(tasks2) ? tasks2.length : 1} job(s) for task ${name}`
              );
              const toProcess = Array.isArray(tasks2) ? tasks2 : [tasks2];
              await Promise.all(
                toProcess.map(
                  (task) => process2({ data: task.data }).then(async () => {
                    await adapterSuccess?.({ task });
                    await success?.({ data: task.data });
                  }).catch(async (err) => {
                    log7.verbose(`error processing task ${name}:`, err);
                    try {
                      await adapterFailure?.({ task, err });
                      await failure?.({ data: task.data, err });
                    } catch (err2) {
                      log7.error(err2);
                    }
                  })
                )
              );
              log7.verbose(
                `processed ${toProcess.length} job(s) for task ${name}`
              );
            } catch (err) {
              log7.error(`error occurred while processing task ${name}:`, err);
            }
          }
        } finally {
          executing = false;
        }
      }, pollIntervall);
      return timeout;
    }
  };
}

// src/task/persisted-task.ts
function createPersistedTask({
  data,
  name
}) {
  return {
    data,
    id: randomId("task"),
    name,
    created: Date.now(),
    failedLast: null,
    failedNr: null,
    failedError: null
  };
}

// src/task/database-task.ts
var tasks = (0, import_sqlite_core.sqliteTable)("tasks", {
  id: (0, import_sqlite_core.text)("id").primaryKey(),
  name: (0, import_sqlite_core.text)("name").notNull(),
  data: (0, import_sqlite_core.text)("data", { mode: "json" }),
  created: (0, import_sqlite_core.int)("created").notNull(),
  failedLast: (0, import_sqlite_core.int)("failed_last"),
  failedNr: (0, import_sqlite_core.int)("failed_nr"),
  failedError: (0, import_sqlite_core.text)("failed_error")
});
var _ = tasks.$inferSelect;
function createDatabaseAdapter(database3) {
  return {
    async enqueue({ data, name }) {
      const persistedTask = createPersistedTask({ data, name });
      await database3.insert(tasks).values(persistedTask);
    },
    async fetch({ name: taskName, batchSize, maxRetries, retryIntervall }) {
      const tasks2 = await database3.query.tasks.findMany({
        limit: batchSize,
        orderBy: ({ created }) => (0, import_drizzle_orm5.asc)(created),
        where: ({ failedNr, failedLast, name }) => (0, import_drizzle_orm3.and)(
          (0, import_drizzle_orm3.or)((0, import_drizzle_orm3.lte)(failedNr, maxRetries), (0, import_drizzle_orm6.isNull)(failedNr)),
          (0, import_drizzle_orm3.or)(
            (0, import_drizzle_orm3.lte)(failedLast, Date.now() - retryIntervall),
            (0, import_drizzle_orm6.isNull)(failedLast)
          ),
          (0, import_drizzle_orm4.eq)(name, taskName)
        )
      });
      return tasks2;
    },
    async success({ task }) {
      await database3.delete(tasks).where((0, import_drizzle_orm4.eq)(tasks.id, task.id));
    },
    async failure({ task, err }) {
      await database3.update(tasks).set({
        failedLast: Date.now(),
        failedNr: (task.failedNr ?? 0) + 1,
        failedError: JSON.stringify({
          instance: err.constructor.name,
          message: err.message,
          stack: err.stack
        })
      }).where((0, import_drizzle_orm4.eq)(tasks.id, task.id));
    }
  };
}
function defineDatabaseTask(database3, opts) {
  return defineTaskWithStorageAdapter(
    createDatabaseAdapter(database3),
    opts
  );
}

// src/task/inmemory-task.ts
var inmemoryTasks = /* @__PURE__ */ new Map();
var InmemoryAdapter = {
  async enqueue({ data, name }) {
    const persistedTask = createPersistedTask({ data, name });
    inmemoryTasks.set(persistedTask.id, persistedTask);
  },
  async fetch({ name, batchSize, maxRetries, retryIntervall }) {
    const result = Array.from(inmemoryTasks.values()).filter(
      ({ name: taskName, failedNr, failedLast }) => taskName === name && (failedNr ?? 0) <= maxRetries && (failedLast ?? 0) <= Date.now() - retryIntervall
    ).slice(0, batchSize);
    return result;
  },
  async success({ task }) {
    inmemoryTasks.delete(task.id);
  },
  async failure({ task, err }) {
    inmemoryTasks.set(task.id, {
      ...task,
      failedNr: (task.failedNr ?? 0) + 1,
      failedLast: Date.now(),
      failedError: JSON.stringify({
        instance: err.constructor.name,
        message: err.message,
        stack: err.stack
      })
    });
  }
};
function defineInmemoryTask(opts) {
  return defineTaskWithStorageAdapter(
    InmemoryAdapter,
    opts
  );
}

// src/admin/config.ts
var config = {
  verbose: 3,
  adminBasePath: "/_"
};

// src/admin/admin.ts
var import_express4 = __toESM(require("express"));

// src/admin/column.ts
function columnType(sqliteType) {
  switch (sqliteType) {
    case "INTEGER":
      return "number";
    case "REAL":
      return "number";
    case "TEXT":
      return "string";
    case "BLOB":
      return "Buffer";
    case "TIMESTAMP":
      return "Date";
    default:
      return "any";
  }
}
function renderValue(value, tsType) {
  if (value === null) {
    return "NULL";
  }
  switch (tsType) {
    case "number":
      return String(value);
    case "string":
      return value;
    case "Buffer":
      return value.toString("hex");
    case "Date":
      return new Date(value).toISOString();
    default:
      return value.toString();
  }
}

// src/admin/database/routes/[table]/components/table-row.tsx
var import_jsx_runtime = require("@kitajs/html/jsx-runtime");
function TableRow({ editing, tableName, columns, row }) {
  return /* @__PURE__ */ (0, import_jsx_runtime.jsxs)("tr", { children: [
    /* @__PURE__ */ (0, import_jsx_runtime.jsx)("td", { class: "px-1 py-0 text-sm invisible w-16", children: /* @__PURE__ */ (0, import_jsx_runtime.jsx)(
      "button",
      {
        class: "btn btn-xs mr-2",
        type: "submit",
        "hx-target": "closest tr",
        "hx-swap": "outerHTML",
        "hx-get": `${config.adminBasePath}/database/${tableName}/edit?row=${encodeURIComponent(JSON.stringify(row))}`,
        "hx-trigger": "click from:closest tr"
      }
    ) }),
    columns.map((column) => {
      const tsType = columnType(column.type);
      const value = row[column.name];
      const formattedValue = renderValue(value, tsType);
      return /* @__PURE__ */ (0, import_jsx_runtime.jsx)(
        "td",
        {
          safe: true,
          "data-type": tsType,
          class: "border border-neutral px-2 py-1 max-w-64 min-w-32 truncate",
          children: formattedValue
        }
      );
    }),
    " "
  ] });
}

// src/admin/database/routes/[table]/edit.tsx
var import_drizzle_orm7 = require("drizzle-orm");
var import_jsx_runtime2 = require("@kitajs/html/jsx-runtime");
var POST = async ({ req, res }) => {
  const tableName = req.params.table;
  const db = res.locals.database;
  const { __row, ...updatedData } = req.body;
  const columns = db.all(
    import_drizzle_orm7.sql`SELECT * FROM pragma_table_info(${tableName})`
  );
  const newData = {};
  for (const [key, value] of Object.entries(updatedData)) {
    const column = columns.find((column2) => column2.name === key);
    if (!column) {
      throw new Error(`Column ${key} not found in table ${tableName}`);
    }
    newData[key] = value;
    if (column.type === "INTEGER") {
      newData[key] = Number.parseInt(value);
    } else if (column.type === "REAL") {
      newData[key] = Number.parseFloat(value);
    }
  }
  config.verbose >= 1 || console.log("[admin] [database]", "saving row", newData);
  const oldRow = JSON.parse(__row);
  const setClause = Object.entries(newData).map(([column, value]) => import_drizzle_orm7.sql`${import_drizzle_orm7.sql.identifier(column)} = ${value}`).reduce((acc, curr) => import_drizzle_orm7.sql`${acc}, ${curr}`);
  const whereClause = Object.entries(oldRow).map(([column, value]) => import_drizzle_orm7.sql`${import_drizzle_orm7.sql.identifier(column)} = ${value}`).reduce((acc, curr) => import_drizzle_orm7.sql`${acc} AND ${curr}`);
  db.run(
    import_drizzle_orm7.sql`UPDATE ${import_drizzle_orm7.sql.identifier(tableName)} SET ${setClause} WHERE ${whereClause}`
  );
  config.verbose >= 1 || console.log("[admin] [database]", "row saved");
  return /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(TableRow, { tableName, columns, row: newData });
};
function estimateNrOfRows(value) {
  return String(Math.max(Math.round(String(value).length / 30), 1));
}
var GET = async ({ req, res }) => {
  const tableName = req.params.table;
  const rowData = JSON.parse(decodeURIComponent(req.query.row));
  const db = res.locals.database;
  const columns = db.all(
    import_drizzle_orm7.sql`SELECT * FROM pragma_table_info(${tableName})`
  );
  return /* @__PURE__ */ (0, import_jsx_runtime2.jsxs)("tr", { children: [
    /* @__PURE__ */ (0, import_jsx_runtime2.jsxs)("td", { class: "w-16 flex items-center justify-center", children: [
      /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(
        "button",
        {
          class: "btn btn-xs btn-primary",
          type: "submit",
          "hx-post": `${config.adminBasePath}/database/${tableName}/edit`,
          "hx-include": "closest tr",
          children: "Save"
        }
      ),
      /* @__PURE__ */ (0, import_jsx_runtime2.jsx)("input", { type: "hidden", name: "__row", value: JSON.stringify(rowData) })
    ] }),
    columns.map((column) => {
      const tsType = columnType(column.type);
      const value = rowData[column.name];
      const safeFormattedValue = renderValue(value, tsType);
      return /* @__PURE__ */ (0, import_jsx_runtime2.jsxs)(
        "td",
        {
          "data-type": tsType,
          class: "border border-neutral px-2 py-1 max-w-32 relative",
          children: [
            safeFormattedValue,
            /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(
              "textarea",
              {
                class: "top-0 left-0 w-full py-1 pl-2 m-0 bg-base-200 absolute",
                rows: estimateNrOfRows(value),
                name: column.name,
                safe: true,
                children: value
              }
            )
          ]
        }
      );
    }),
    /* @__PURE__ */ (0, import_jsx_runtime2.jsx)("td", { children: /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(
      "button",
      {
        class: "btn btn-xs ml-1",
        type: "reset",
        "hx-get": `${config.adminBasePath}/database/${tableName}/row?row=${encodeURIComponent(JSON.stringify(rowData))}`,
        children: "Cancel"
      }
    ) })
  ] });
};

// src/admin/layout.tsx
var import_jsx_runtime3 = require("@kitajs/html/jsx-runtime");
function DatabaseIcon() {
  return /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)(
    "svg",
    {
      xmlns: "http://www.w3.org/2000/svg",
      width: "24",
      height: "24",
      viewBox: "0 0 24 24",
      fill: "none",
      stroke: "currentColor",
      "stroke-width": "1",
      "stroke-linecap": "round",
      "stroke-linejoin": "round",
      class: "size-5",
      children: [
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("title", { children: "Database" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("ellipse", { cx: "12", cy: "5", rx: "9", ry: "3" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("path", { d: "M3 5V19A9 3 0 0 0 21 19V5" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("path", { d: "M3 12A9 3 0 0 0 21 12" })
      ]
    }
  );
}
function MediaIcon() {
  return /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)(
    "svg",
    {
      xmlns: "http://www.w3.org/2000/svg",
      width: "24",
      height: "24",
      viewBox: "0 0 24 24",
      fill: "none",
      stroke: "currentColor",
      "stroke-width": "1",
      "stroke-linecap": "round",
      "stroke-linejoin": "round",
      class: "lucide lucide-image",
      children: [
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("title", { children: "Media" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("rect", { width: "18", height: "18", x: "3", y: "3", rx: "2", ry: "2" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("circle", { cx: "9", cy: "9", r: "2" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("path", { d: "m21 15-3.086-3.086a2 2 0 0 0-2.828 0L6 21" })
      ]
    }
  );
}
function TaskQueueIcons() {
  return /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)(
    "svg",
    {
      xmlns: "http://www.w3.org/2000/svg",
      width: "24",
      height: "24",
      viewBox: "0 0 24 24",
      fill: "none",
      stroke: "currentColor",
      "stroke-width": "1",
      "stroke-linecap": "round",
      "stroke-linejoin": "round",
      class: "lucide lucide-rows-2",
      children: [
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("title", { children: "Task Queues" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("rect", { width: "18", height: "18", x: "3", y: "3", rx: "2" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("path", { d: "M3 12h18" })
      ]
    }
  );
}
function MainNavigation(props) {
  return /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("ul", { class: "menu bg-base-200 rounded-box w-56 mb-2", children: [
    /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("li", { children: /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)(
      "a",
      {
        class: props.active === "database" ? "active" : "",
        href: `${config.adminBasePath}/database`,
        children: [
          /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(DatabaseIcon, {}),
          "Database"
        ]
      }
    ) }),
    /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("li", { class: "disabled", children: /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("a", { href: `${config.adminBasePath}/users`, children: [
      /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(TaskQueueIcons, {}),
      "Task Queues"
    ] }) }),
    /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("li", { class: "disabled", children: /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("a", { href: `${config.adminBasePath}/media`, children: [
      /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(MediaIcon, {}),
      "Media Uploads"
    ] }) })
  ] });
}
function SubNavigation(props) {
  return /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("ul", { class: "menu menu-sm", children: props.items.map((item) => {
    const safeIcon = item.icon;
    const active = props.path.endsWith(item.href.split("/").pop() ?? "");
    const safeLabel = item.label;
    return /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("li", { children: [
      item.icon && /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("span", { children: safeIcon }),
      /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("a", { class: active ? "active" : "", href: item.href, children: safeLabel })
    ] });
  }) });
}
function AdminLayout(props) {
  return /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)(import_jsx_runtime3.Fragment, { children: [
    "<!doctype html>",
    /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("html", { lang: "en", "data-theme": "auto", children: [
      /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("head", { children: [
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("meta", { charset: "UTF-8" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(
          "meta",
          {
            name: "viewport",
            content: "width=device-width, initial-scale=1.0"
          }
        ),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("title", { children: props.title || "plainweb" }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(
          "meta",
          {
            name: "description",
            content: props.description || "Build web app with less complexity and more joy."
          }
        ),
        process.env.BIN_ADMIN_TESTING ? /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("link", { rel: "stylesheet", href: "/public/output.css" }) : /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("style", { children: `/*! tailwindcss v3.4.10 | MIT License | https://tailwindcss.com*/*,:after,:before{box-sizing:border-box;border:0 solid #e5e7eb}:after,:before{--tw-content:""}:host,html{line-height:1.5;-webkit-text-size-adjust:100%;-moz-tab-size:4;-o-tab-size:4;tab-size:4;font-family:ui-sans-serif,system-ui,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji;font-feature-settings:normal;font-variation-settings:normal;-webkit-tap-highlight-color:transparent}body{margin:0;line-height:inherit}hr{height:0;color:inherit;border-top-width:1px}abbr:where([title]){-webkit-text-decoration:underline dotted;text-decoration:underline dotted}h1,h2,h3,h4,h5,h6{font-size:inherit;font-weight:inherit}a{color:inherit;text-decoration:inherit}b,strong{font-weight:bolder}code,kbd,pre,samp{font-family:ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,Liberation Mono,Courier New,monospace;font-feature-settings:normal;font-variation-settings:normal;font-size:1em}small{font-size:80%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sub{bottom:-.25em}sup{top:-.5em}table{text-indent:0;border-color:inherit;border-collapse:collapse}button,input,optgroup,select,textarea{font-family:inherit;font-feature-settings:inherit;font-variation-settings:inherit;font-size:100%;font-weight:inherit;line-height:inherit;letter-spacing:inherit;color:inherit;margin:0;padding:0}button,select{text-transform:none}button,input:where([type=button]),input:where([type=reset]),input:where([type=submit]){-webkit-appearance:button;background-color:transparent;background-image:none}:-moz-focusring{outline:auto}:-moz-ui-invalid{box-shadow:none}progress{vertical-align:baseline}::-webkit-inner-spin-button,::-webkit-outer-spin-button{height:auto}[type=search]{-webkit-appearance:textfield;outline-offset:-2px}::-webkit-search-decoration{-webkit-appearance:none}::-webkit-file-upload-button{-webkit-appearance:button;font:inherit}summary{display:list-item}blockquote,dd,dl,figure,h1,h2,h3,h4,h5,h6,hr,p,pre{margin:0}fieldset{margin:0}fieldset,legend{padding:0}menu,ol,ul{list-style:none;margin:0;padding:0}dialog{padding:0}textarea{resize:vertical}input::-moz-placeholder,textarea::-moz-placeholder{opacity:1;color:#9ca3af}input::placeholder,textarea::placeholder{opacity:1;color:#9ca3af}[role=button],button{cursor:pointer}:disabled{cursor:default}audio,canvas,embed,iframe,img,object,svg,video{display:block;vertical-align:middle}img,video{max-width:100%;height:auto}[hidden]{display:none}:root,[data-theme]{background-color:var(--fallback-b1,oklch(var(--b1)/1));color:var(--fallback-bc,oklch(var(--bc)/1))}@supports not (color:oklch(0% 0 0)){:root{color-scheme:light;--fallback-p:#491eff;--fallback-pc:#d4dbff;--fallback-s:#ff41c7;--fallback-sc:#fff9fc;--fallback-a:#00cfbd;--fallback-ac:#00100d;--fallback-n:#2b3440;--fallback-nc:#d7dde4;--fallback-b1:#fff;--fallback-b2:#e5e6e6;--fallback-b3:#e5e6e6;--fallback-bc:#1f2937;--fallback-in:#00b3f0;--fallback-inc:#000;--fallback-su:#00ca92;--fallback-suc:#000;--fallback-wa:#ffc22d;--fallback-wac:#000;--fallback-er:#ff6f70;--fallback-erc:#000}@media (prefers-color-scheme:dark){:root{color-scheme:dark;--fallback-p:#7582ff;--fallback-pc:#050617;--fallback-s:#ff71cf;--fallback-sc:#190211;--fallback-a:#00c7b5;--fallback-ac:#000e0c;--fallback-n:#2a323c;--fallback-nc:#a6adbb;--fallback-b1:#1d232a;--fallback-b2:#191e24;--fallback-b3:#15191e;--fallback-bc:#a6adbb;--fallback-in:#00b3f0;--fallback-inc:#000;--fallback-su:#00ca92;--fallback-suc:#000;--fallback-wa:#ffc22d;--fallback-wac:#000;--fallback-er:#ff6f70;--fallback-erc:#000}}}html{-webkit-tap-highlight-color:transparent}*{scrollbar-color:color-mix(in oklch,currentColor 35%,transparent) transparent}:hover{scrollbar-color:color-mix(in oklch,currentColor 60%,transparent) transparent}:root{color-scheme:light;--in:72.06% 0.191 231.6;--su:64.8% 0.150 160;--wa:84.71% 0.199 83.87;--er:71.76% 0.221 22.18;--pc:89.824% 0.06192 275.75;--ac:15.352% 0.0368 183.61;--inc:0% 0 0;--suc:0% 0 0;--wac:0% 0 0;--erc:0% 0 0;--rounded-box:1rem;--rounded-btn:0.5rem;--rounded-badge:1.9rem;--animation-btn:0.25s;--animation-input:.2s;--btn-focus-scale:0.95;--border-btn:1px;--tab-border:1px;--tab-radius:0.5rem;--p:49.12% 0.3096 275.75;--s:69.71% 0.329 342.55;--sc:98.71% 0.0106 342.55;--a:76.76% 0.184 183.61;--n:32.1785% 0.02476 255.701624;--nc:89.4994% 0.011585 252.096176;--b1:100% 0 0;--b2:96.1151% 0 0;--b3:92.4169% 0.00108 197.137559;--bc:27.8078% 0.029596 256.847952}@media (prefers-color-scheme:dark){:root{color-scheme:dark;--in:72.06% 0.191 231.6;--su:64.8% 0.150 160;--wa:84.71% 0.199 83.87;--er:71.76% 0.221 22.18;--pc:13.138% 0.0392 275.75;--sc:14.96% 0.052 342.55;--ac:14.902% 0.0334 183.61;--inc:0% 0 0;--suc:0% 0 0;--wac:0% 0 0;--erc:0% 0 0;--rounded-box:1rem;--rounded-btn:0.5rem;--rounded-badge:1.9rem;--animation-btn:0.25s;--animation-input:.2s;--btn-focus-scale:0.95;--border-btn:1px;--tab-border:1px;--tab-radius:0.5rem;--p:65.69% 0.196 275.75;--s:74.8% 0.26 342.55;--a:74.51% 0.167 183.61;--n:31.3815% 0.021108 254.139175;--nc:74.6477% 0.0216 264.435964;--b1:25.3267% 0.015896 252.417568;--b2:23.2607% 0.013807 253.100675;--b3:21.1484% 0.01165 254.087939;--bc:74.6477% 0.0216 264.435964}}[data-theme=light]{color-scheme:light;--in:72.06% 0.191 231.6;--su:64.8% 0.150 160;--wa:84.71% 0.199 83.87;--er:71.76% 0.221 22.18;--pc:89.824% 0.06192 275.75;--ac:15.352% 0.0368 183.61;--inc:0% 0 0;--suc:0% 0 0;--wac:0% 0 0;--erc:0% 0 0;--rounded-box:1rem;--rounded-btn:0.5rem;--rounded-badge:1.9rem;--animation-btn:0.25s;--animation-input:.2s;--btn-focus-scale:0.95;--border-btn:1px;--tab-border:1px;--tab-radius:0.5rem;--p:49.12% 0.3096 275.75;--s:69.71% 0.329 342.55;--sc:98.71% 0.0106 342.55;--a:76.76% 0.184 183.61;--n:32.1785% 0.02476 255.701624;--nc:89.4994% 0.011585 252.096176;--b1:100% 0 0;--b2:96.1151% 0 0;--b3:92.4169% 0.00108 197.137559;--bc:27.8078% 0.029596 256.847952}[data-theme=dark]{color-scheme:dark;--in:72.06% 0.191 231.6;--su:64.8% 0.150 160;--wa:84.71% 0.199 83.87;--er:71.76% 0.221 22.18;--pc:13.138% 0.0392 275.75;--sc:14.96% 0.052 342.55;--ac:14.902% 0.0334 183.61;--inc:0% 0 0;--suc:0% 0 0;--wac:0% 0 0;--erc:0% 0 0;--rounded-box:1rem;--rounded-btn:0.5rem;--rounded-badge:1.9rem;--animation-btn:0.25s;--animation-input:.2s;--btn-focus-scale:0.95;--border-btn:1px;--tab-border:1px;--tab-radius:0.5rem;--p:65.69% 0.196 275.75;--s:74.8% 0.26 342.55;--a:74.51% 0.167 183.61;--n:31.3815% 0.021108 254.139175;--nc:74.6477% 0.0216 264.435964;--b1:25.3267% 0.015896 252.417568;--b2:23.2607% 0.013807 253.100675;--b3:21.1484% 0.01165 254.087939;--bc:74.6477% 0.0216 264.435964}*,:after,:before{--tw-border-spacing-x:0;--tw-border-spacing-y:0;--tw-translate-x:0;--tw-translate-y:0;--tw-rotate:0;--tw-skew-x:0;--tw-skew-y:0;--tw-scale-x:1;--tw-scale-y:1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness:proximity;--tw-gradient-from-position: ;--tw-gradient-via-position: ;--tw-gradient-to-position: ;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width:0px;--tw-ring-offset-color:#fff;--tw-ring-color:rgba(59,130,246,.5);--tw-ring-offset-shadow:0 0 #0000;--tw-ring-shadow:0 0 #0000;--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: ;--tw-contain-size: ;--tw-contain-layout: ;--tw-contain-paint: ;--tw-contain-style: }::backdrop{--tw-border-spacing-x:0;--tw-border-spacing-y:0;--tw-translate-x:0;--tw-translate-y:0;--tw-rotate:0;--tw-skew-x:0;--tw-skew-y:0;--tw-scale-x:1;--tw-scale-y:1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness:proximity;--tw-gradient-from-position: ;--tw-gradient-via-position: ;--tw-gradient-to-position: ;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width:0px;--tw-ring-offset-color:#fff;--tw-ring-color:rgba(59,130,246,.5);--tw-ring-offset-shadow:0 0 #0000;--tw-ring-shadow:0 0 #0000;--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: ;--tw-contain-size: ;--tw-contain-layout: ;--tw-contain-paint: ;--tw-contain-style: }@media (hover:hover){.label a:hover{--tw-text-opacity:1;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)))}.menu li>:not(ul,.menu-title,details,.btn).active,.menu li>:not(ul,.menu-title,details,.btn):active,.menu li>details>summary:active{--tw-bg-opacity:1;background-color:var(--fallback-n,oklch(var(--n)/var(--tw-bg-opacity)));--tw-text-opacity:1;color:var(--fallback-nc,oklch(var(--nc)/var(--tw-text-opacity)))}.table tr.hover:hover,.table tr.hover:nth-child(2n):hover{--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)))}}.btn{display:inline-flex;height:3rem;min-height:3rem;flex-shrink:0;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;user-select:none;flex-wrap:wrap;align-items:center;justify-content:center;border-radius:var(--rounded-btn,.5rem);border-color:transparent;border-color:oklch(var(--btn-color,var(--b2))/var(--tw-border-opacity));padding-left:1rem;padding-right:1rem;text-align:center;font-size:.875rem;line-height:1em;gap:.5rem;font-weight:600;text-decoration-line:none;transition-duration:.2s;transition-timing-function:cubic-bezier(0,0,.2,1);border-width:var(--border-btn,1px);transition-property:color,background-color,border-color,opacity,box-shadow,transform;--tw-text-opacity:1;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)));--tw-shadow:0 1px 2px 0 rgba(0,0,0,.05);--tw-shadow-colored:0 1px 2px 0 var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow);outline-color:var(--fallback-bc,oklch(var(--bc)/1));background-color:oklch(var(--btn-color,var(--b2))/var(--tw-bg-opacity));--tw-bg-opacity:1;--tw-border-opacity:1}.btn-disabled,.btn:disabled,.btn[disabled]{pointer-events:none}:where(.btn:is(input[type=checkbox])),:where(.btn:is(input[type=radio])){width:auto;-webkit-appearance:none;-moz-appearance:none;appearance:none}.btn:is(input[type=checkbox]):after,.btn:is(input[type=radio]):after{--tw-content:attr(aria-label);content:var(--tw-content)}@media (hover:hover){.btm-nav>.disabled:hover,.btm-nav>[disabled]:hover{pointer-events:none;--tw-border-opacity:0;background-color:var(--fallback-n,oklch(var(--n)/var(--tw-bg-opacity)));--tw-bg-opacity:0.1;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)));--tw-text-opacity:0.2}.btn:hover{--tw-border-opacity:1;border-color:var(--fallback-b3,oklch(var(--b3)/var(--tw-border-opacity)));--tw-bg-opacity:1;background-color:var(--fallback-b3,oklch(var(--b3)/var(--tw-bg-opacity)))}@supports (color:color-mix(in oklab,black,black)){.btn:hover{background-color:color-mix(in oklab,oklch(var(--btn-color,var(--b2))/var(--tw-bg-opacity,1)) 90%,#000);border-color:color-mix(in oklab,oklch(var(--btn-color,var(--b2))/var(--tw-border-opacity,1)) 90%,#000)}}@supports not (color:oklch(0% 0 0)){.btn:hover{background-color:var(--btn-color,var(--fallback-b2));border-color:var(--btn-color,var(--fallback-b2))}}.btn.glass:hover{--glass-opacity:25%;--glass-border-opacity:15%}.btn-ghost:hover{border-color:transparent}@supports (color:oklch(0% 0 0)){.btn-ghost:hover{background-color:var(--fallback-bc,oklch(var(--bc)/.2))}}.btn-outline.btn-primary:hover{--tw-text-opacity:1;color:var(--fallback-pc,oklch(var(--pc)/var(--tw-text-opacity)))}@supports (color:color-mix(in oklab,black,black)){.btn-outline.btn-primary:hover{background-color:color-mix(in oklab,var(--fallback-p,oklch(var(--p)/1)) 90%,#000);border-color:color-mix(in oklab,var(--fallback-p,oklch(var(--p)/1)) 90%,#000)}}.btn-disabled:hover,.btn:disabled:hover,.btn[disabled]:hover{--tw-border-opacity:0;background-color:var(--fallback-n,oklch(var(--n)/var(--tw-bg-opacity)));--tw-bg-opacity:0.2;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)));--tw-text-opacity:0.2}@supports (color:color-mix(in oklab,black,black)){.btn:is(input[type=checkbox]:checked):hover,.btn:is(input[type=radio]:checked):hover{background-color:color-mix(in oklab,var(--fallback-p,oklch(var(--p)/1)) 90%,#000);border-color:color-mix(in oklab,var(--fallback-p,oklch(var(--p)/1)) 90%,#000)}}:where(.menu li:not(.menu-title,.disabled)>:not(ul,details,.menu-title)):not(.active,.btn):hover,:where(.menu li:not(.menu-title,.disabled)>details>summary:not(.menu-title)):not(.active,.btn):hover{cursor:pointer;outline:2px solid transparent;outline-offset:2px}@supports (color:oklch(0% 0 0)){:where(.menu li:not(.menu-title,.disabled)>:not(ul,details,.menu-title)):not(.active,.btn):hover,:where(.menu li:not(.menu-title,.disabled)>details>summary:not(.menu-title)):not(.active,.btn):hover{background-color:var(--fallback-bc,oklch(var(--bc)/.1))}}}.label{display:flex;-webkit-user-select:none;-moz-user-select:none;user-select:none;align-items:center;justify-content:space-between;padding:.5rem .25rem}.input{flex-shrink:1;-webkit-appearance:none;-moz-appearance:none;appearance:none;height:3rem;padding-left:1rem;padding-right:1rem;font-size:1rem;line-height:2;line-height:1.5rem;border-radius:var(--rounded-btn,.5rem);border-width:1px;border-color:transparent;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.input-md[type=number]::-webkit-inner-spin-button,.input[type=number]::-webkit-inner-spin-button{margin-top:-1rem;margin-bottom:-1rem;margin-inline-end:-1rem}.join{display:inline-flex;align-items:stretch;border-radius:var(--rounded-btn,.5rem)}.join :where(.join-item){border-start-end-radius:0;border-end-end-radius:0;border-end-start-radius:0;border-start-start-radius:0}.join .join-item:not(:first-child):not(:last-child),.join :not(:first-child):not(:last-child) .join-item{border-start-end-radius:0;border-end-end-radius:0;border-end-start-radius:0;border-start-start-radius:0}.join .join-item:first-child:not(:last-child),.join :first-child:not(:last-child) .join-item{border-start-end-radius:0;border-end-end-radius:0}.join .dropdown .join-item:first-child:not(:last-child),.join :first-child:not(:last-child) .dropdown .join-item{border-start-end-radius:inherit;border-end-end-radius:inherit}.join :where(.join-item:first-child:not(:last-child)),.join :where(:first-child:not(:last-child) .join-item){border-end-start-radius:inherit;border-start-start-radius:inherit}.join .join-item:last-child:not(:first-child),.join :last-child:not(:first-child) .join-item{border-end-start-radius:0;border-start-start-radius:0}.join :where(.join-item:last-child:not(:first-child)),.join :where(:last-child:not(:first-child) .join-item){border-start-end-radius:inherit;border-end-end-radius:inherit}@supports not selector(:has(*)){:where(.join *){border-radius:inherit}}@supports selector(:has(*)){:where(.join :has(.join-item)){border-radius:inherit}}.link{cursor:pointer;text-decoration-line:underline}.menu{display:flex;flex-direction:column;flex-wrap:wrap;font-size:.875rem;line-height:1.25rem;padding:.5rem}.menu :where(li ul){position:relative;white-space:nowrap;margin-inline-start:1rem;padding-inline-start:.5rem}.menu :where(li:not(.menu-title)>:not(ul,details,.menu-title,.btn)),.menu :where(li:not(.menu-title)>details>summary:not(.menu-title)){display:grid;grid-auto-flow:column;align-content:flex-start;align-items:center;gap:.5rem;grid-auto-columns:minmax(auto,max-content) auto max-content;-webkit-user-select:none;-moz-user-select:none;user-select:none}.menu li.disabled{cursor:not-allowed;-webkit-user-select:none;-moz-user-select:none;user-select:none;color:var(--fallback-bc,oklch(var(--bc)/.3))}.menu :where(li>.menu-dropdown:not(.menu-dropdown-show)){display:none}:where(.menu li){position:relative;display:flex;flex-shrink:0;flex-direction:column;flex-wrap:wrap;align-items:stretch}:where(.menu li) .badge{justify-self:end}.navbar{display:flex;align-items:center;padding:var(--navbar-padding,.5rem);min-height:4rem;width:100%}:where(.navbar>:not(script,style)){display:inline-flex;align-items:center}.select{display:inline-flex;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;user-select:none;-webkit-appearance:none;-moz-appearance:none;appearance:none;height:3rem;min-height:3rem;padding-inline-start:1rem;padding-inline-end:2.5rem;font-size:.875rem;line-height:1.25rem;line-height:2;border-radius:var(--rounded-btn,.5rem);border-width:1px;border-color:transparent;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)));background-image:linear-gradient(45deg,transparent 50%,currentColor 0),linear-gradient(135deg,currentColor 50%,transparent 0);background-position:calc(100% - 20px) calc(1px + 50%),calc(100% - 16.1px) calc(1px + 50%);background-size:4px 4px,4px 4px;background-repeat:no-repeat}.select[multiple]{height:auto}.stack{display:inline-grid;place-items:center;align-items:flex-end}.stack>*{grid-column-start:1;grid-row-start:1;transform:translateY(10%) scale(.9);z-index:1;width:100%;opacity:.6}.stack>:nth-child(2){transform:translateY(5%) scale(.95);z-index:2;opacity:.8}.stack>:first-child{transform:translateY(0) scale(1);z-index:3;opacity:1}.stat{display:inline-grid;grid-template-columns:repeat(1,1fr);-moz-column-gap:1rem;column-gap:1rem;border-color:var(--fallback-bc,oklch(var(--bc)/var(--tw-border-opacity)));--tw-border-opacity:0.1;padding:1rem 1.5rem}.stat,.table{width:100%}.table{position:relative;border-radius:var(--rounded-box,1rem);text-align:left;font-size:.875rem;line-height:1.25rem}.table :where(.table-pin-rows thead tr){position:sticky;top:0;z-index:1;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.table :where(.table-pin-rows tfoot tr){position:sticky;bottom:0;z-index:1;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.table :where(.table-pin-cols tr th){position:sticky;left:0;right:0;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.textarea{min-height:3rem;flex-shrink:1;padding:.5rem 1rem;font-size:.875rem;line-height:1.25rem;line-height:2;border-radius:var(--rounded-btn,.5rem);border-width:1px;border-color:transparent;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.btm-nav>:where(.active){border-top-width:2px;--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.btm-nav>.disabled,.btm-nav>[disabled]{pointer-events:none;--tw-border-opacity:0;background-color:var(--fallback-n,oklch(var(--n)/var(--tw-bg-opacity)));--tw-bg-opacity:0.1;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)));--tw-text-opacity:0.2}.btm-nav>* .label{font-size:1rem;line-height:1.5rem}@media (prefers-reduced-motion:no-preference){.btn{animation:button-pop var(--animation-btn,.25s) ease-out}}.btn:active:focus,.btn:active:hover{animation:button-pop 0s ease-out;transform:scale(var(--btn-focus-scale,.97))}@supports not (color:oklch(0% 0 0)){.btn{background-color:var(--btn-color,var(--fallback-b2));border-color:var(--btn-color,var(--fallback-b2))}.btn-primary{--btn-color:var(--fallback-p)}}@supports (color:color-mix(in oklab,black,black)){.btn-outline.btn-primary.btn-active{background-color:color-mix(in oklab,var(--fallback-p,oklch(var(--p)/1)) 90%,#000);border-color:color-mix(in oklab,var(--fallback-p,oklch(var(--p)/1)) 90%,#000)}}.btn:focus-visible{outline-style:solid;outline-width:2px;outline-offset:2px}.btn-primary{--tw-text-opacity:1;color:var(--fallback-pc,oklch(var(--pc)/var(--tw-text-opacity)));outline-color:var(--fallback-p,oklch(var(--p)/1))}@supports (color:oklch(0% 0 0)){.btn-primary{--btn-color:var(--p)}}.btn.glass{--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow);outline-color:currentColor}.btn.glass.btn-active{--glass-opacity:25%;--glass-border-opacity:15%}.btn-ghost{border-width:1px;border-color:transparent;background-color:transparent;color:currentColor;--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow);outline-color:currentColor}.btn-ghost.btn-active{border-color:transparent;background-color:var(--fallback-bc,oklch(var(--bc)/.2))}.btn-outline.btn-primary{--tw-text-opacity:1;color:var(--fallback-p,oklch(var(--p)/var(--tw-text-opacity)))}.btn-outline.btn-primary.btn-active{--tw-text-opacity:1;color:var(--fallback-pc,oklch(var(--pc)/var(--tw-text-opacity)))}.btn.btn-disabled,.btn:disabled,.btn[disabled]{--tw-border-opacity:0;background-color:var(--fallback-n,oklch(var(--n)/var(--tw-bg-opacity)));--tw-bg-opacity:0.2;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)));--tw-text-opacity:0.2}.btn:is(input[type=checkbox]:checked),.btn:is(input[type=radio]:checked){--tw-border-opacity:1;border-color:var(--fallback-p,oklch(var(--p)/var(--tw-border-opacity)));--tw-bg-opacity:1;background-color:var(--fallback-p,oklch(var(--p)/var(--tw-bg-opacity)));--tw-text-opacity:1;color:var(--fallback-pc,oklch(var(--pc)/var(--tw-text-opacity)))}.btn:is(input[type=checkbox]:checked):focus-visible,.btn:is(input[type=radio]:checked):focus-visible{outline-color:var(--fallback-p,oklch(var(--p)/1))}@keyframes button-pop{0%{transform:scale(var(--btn-focus-scale,.98))}40%{transform:scale(1.02)}to{transform:scale(1)}}@keyframes checkmark{0%{background-position-y:5px}50%{background-position-y:-2px}to{background-position-y:0}}.input input{--tw-bg-opacity:1;background-color:var(--fallback-p,oklch(var(--p)/var(--tw-bg-opacity)));background-color:transparent}.input input:focus{outline:2px solid transparent;outline-offset:2px}.input[list]::-webkit-calendar-picker-indicator{line-height:1em}.input:focus,.input:focus-within{box-shadow:none;border-color:var(--fallback-bc,oklch(var(--bc)/.2));outline-style:solid;outline-width:2px;outline-offset:2px;outline-color:var(--fallback-bc,oklch(var(--bc)/.2))}.input-disabled,.input:disabled,.input:has(>input[disabled]),.input[disabled]{cursor:not-allowed;--tw-border-opacity:1;border-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-border-opacity)));--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)));color:var(--fallback-bc,oklch(var(--bc)/.4))}.input-disabled::-moz-placeholder,.input:disabled::-moz-placeholder,.input:has(>input[disabled])::-moz-placeholder,.input[disabled]::-moz-placeholder{color:var(--fallback-bc,oklch(var(--bc)/var(--tw-placeholder-opacity)));--tw-placeholder-opacity:0.2}.input-disabled::placeholder,.input:disabled::placeholder,.input:has(>input[disabled])::placeholder,.input[disabled]::placeholder{color:var(--fallback-bc,oklch(var(--bc)/var(--tw-placeholder-opacity)));--tw-placeholder-opacity:0.2}.input:has(>input[disabled])>input[disabled]{cursor:not-allowed}.input::-webkit-date-and-time-value{text-align:inherit}.join>:where(:not(:first-child)){margin-top:0;margin-bottom:0;margin-inline-start:-1px}.join>:where(:not(:first-child)):is(.btn){margin-inline-start:calc(var(--border-btn)*-1)}.link:focus{outline:2px solid transparent;outline-offset:2px}.link:focus-visible{outline:2px solid currentColor;outline-offset:2px}.loading{pointer-events:none;display:inline-block;aspect-ratio:1/1;width:1.5rem;background-color:currentColor;-webkit-mask-size:100%;mask-size:100%;-webkit-mask-repeat:no-repeat;mask-repeat:no-repeat;-webkit-mask-position:center;mask-position:center;-webkit-mask-image:url("data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' width='24' height='24' stroke='%23000'%3E%3Cstyle%3E@keyframes spinner_zKoa{to{transform:rotate(360deg)}}@keyframes spinner_YpZS{0%25{stroke-dasharray:0 150;stroke-dashoffset:0}47.5%25{stroke-dasharray:42 150;stroke-dashoffset:-16}95%25,to{stroke-dasharray:42 150;stroke-dashoffset:-59}}%3C/style%3E%3Cg style='transform-origin:center;animation:spinner_zKoa 2s linear infinite'%3E%3Ccircle cx='12' cy='12' r='9.5' fill='none' stroke-width='3' class='spinner_V8m1' style='stroke-linecap:round;animation:spinner_YpZS 1.5s ease-out infinite'/%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' width='24' height='24' stroke='%23000'%3E%3Cstyle%3E@keyframes spinner_zKoa{to{transform:rotate(360deg)}}@keyframes spinner_YpZS{0%25{stroke-dasharray:0 150;stroke-dashoffset:0}47.5%25{stroke-dasharray:42 150;stroke-dashoffset:-16}95%25,to{stroke-dasharray:42 150;stroke-dashoffset:-59}}%3C/style%3E%3Cg style='transform-origin:center;animation:spinner_zKoa 2s linear infinite'%3E%3Ccircle cx='12' cy='12' r='9.5' fill='none' stroke-width='3' class='spinner_V8m1' style='stroke-linecap:round;animation:spinner_YpZS 1.5s ease-out infinite'/%3E%3C/g%3E%3C/svg%3E")}:where(.menu li:empty){--tw-bg-opacity:1;background-color:var(--fallback-bc,oklch(var(--bc)/var(--tw-bg-opacity)));opacity:.1;margin:.5rem 1rem;height:1px}.menu :where(li ul):before{position:absolute;bottom:.75rem;inset-inline-start:0;top:.75rem;width:1px;--tw-bg-opacity:1;background-color:var(--fallback-bc,oklch(var(--bc)/var(--tw-bg-opacity)));opacity:.1;content:""}.menu :where(li:not(.menu-title)>:not(ul,details,.menu-title,.btn)),.menu :where(li:not(.menu-title)>details>summary:not(.menu-title)){border-radius:var(--rounded-btn,.5rem);padding:.5rem 1rem;text-align:start;transition-property:color,background-color,border-color,text-decoration-color,fill,stroke,opacity,box-shadow,transform,filter,-webkit-backdrop-filter;transition-property:color,background-color,border-color,text-decoration-color,fill,stroke,opacity,box-shadow,transform,filter,backdrop-filter;transition-property:color,background-color,border-color,text-decoration-color,fill,stroke,opacity,box-shadow,transform,filter,backdrop-filter,-webkit-backdrop-filter;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-timing-function:cubic-bezier(0,0,.2,1);transition-duration:.2s;text-wrap:balance}:where(.menu li:not(.menu-title,.disabled)>:not(ul,details,.menu-title)):is(summary):not(.active,.btn):focus-visible,:where(.menu li:not(.menu-title,.disabled)>:not(ul,details,.menu-title)):not(summary,.active,.btn).focus,:where(.menu li:not(.menu-title,.disabled)>:not(ul,details,.menu-title)):not(summary,.active,.btn):focus,:where(.menu li:not(.menu-title,.disabled)>details>summary:not(.menu-title)):is(summary):not(.active,.btn):focus-visible,:where(.menu li:not(.menu-title,.disabled)>details>summary:not(.menu-title)):not(summary,.active,.btn).focus,:where(.menu li:not(.menu-title,.disabled)>details>summary:not(.menu-title)):not(summary,.active,.btn):focus{cursor:pointer;background-color:var(--fallback-bc,oklch(var(--bc)/.1));--tw-text-opacity:1;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)));outline:2px solid transparent;outline-offset:2px}.menu li>:not(ul,.menu-title,details,.btn).active,.menu li>:not(ul,.menu-title,details,.btn):active,.menu li>details>summary:active{--tw-bg-opacity:1;background-color:var(--fallback-n,oklch(var(--n)/var(--tw-bg-opacity)));--tw-text-opacity:1;color:var(--fallback-nc,oklch(var(--nc)/var(--tw-text-opacity)))}.menu :where(li>details>summary)::-webkit-details-marker{display:none}.menu :where(li>.menu-dropdown-toggle):after,.menu :where(li>details>summary):after{justify-self:end;display:block;margin-top:-.5rem;height:.5rem;width:.5rem;transform:rotate(45deg);transition-property:transform,margin-top;transition-duration:.3s;transition-timing-function:cubic-bezier(.4,0,.2,1);content:"";transform-origin:75% 75%;box-shadow:2px 2px;pointer-events:none}.menu :where(li>.menu-dropdown-toggle.menu-dropdown-show):after,.menu :where(li>details[open]>summary):after{transform:rotate(225deg);margin-top:0}.mockup-browser .mockup-browser-toolbar .input{position:relative;margin-left:auto;margin-right:auto;display:block;height:1.75rem;width:24rem;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)));padding-left:2rem;direction:ltr}.mockup-browser .mockup-browser-toolbar .input:before{left:.5rem;aspect-ratio:1/1;height:.75rem;--tw-translate-y:-50%;border-radius:9999px;border-width:2px;border-color:currentColor}.mockup-browser .mockup-browser-toolbar .input:after,.mockup-browser .mockup-browser-toolbar .input:before{content:"";position:absolute;top:50%;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));opacity:.6}.mockup-browser .mockup-browser-toolbar .input:after{left:1.25rem;height:.5rem;--tw-translate-y:25%;--tw-rotate:-45deg;border-radius:9999px;border-width:1px;border-color:currentColor}@keyframes modal-pop{0%{opacity:0}}@keyframes progress-loading{50%{background-position-x:-115%}}@keyframes radiomark{0%{box-shadow:0 0 0 12px var(--fallback-b1,oklch(var(--b1)/1)) inset,0 0 0 12px var(--fallback-b1,oklch(var(--b1)/1)) inset}50%{box-shadow:0 0 0 3px var(--fallback-b1,oklch(var(--b1)/1)) inset,0 0 0 3px var(--fallback-b1,oklch(var(--b1)/1)) inset}to{box-shadow:0 0 0 4px var(--fallback-b1,oklch(var(--b1)/1)) inset,0 0 0 4px var(--fallback-b1,oklch(var(--b1)/1)) inset}}@keyframes rating-pop{0%{transform:translateY(-.125em)}40%{transform:translateY(-.125em)}to{transform:translateY(0)}}.select:focus{box-shadow:none;border-color:var(--fallback-bc,oklch(var(--bc)/.2));outline-style:solid;outline-width:2px;outline-offset:2px;outline-color:var(--fallback-bc,oklch(var(--bc)/.2))}.select-disabled,.select:disabled,.select[disabled]{cursor:not-allowed;--tw-border-opacity:1;border-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-border-opacity)));--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)));color:var(--fallback-bc,oklch(var(--bc)/.4))}.select-disabled::-moz-placeholder,.select:disabled::-moz-placeholder,.select[disabled]::-moz-placeholder{color:var(--fallback-bc,oklch(var(--bc)/var(--tw-placeholder-opacity)));--tw-placeholder-opacity:0.2}.select-disabled::placeholder,.select:disabled::placeholder,.select[disabled]::placeholder{color:var(--fallback-bc,oklch(var(--bc)/var(--tw-placeholder-opacity)));--tw-placeholder-opacity:0.2}.select-multiple,.select[multiple],.select[size].select:not([size="1"]){background-image:none;padding-right:1rem}[dir=rtl] .select{background-position:12px calc(1px + 50%),16px calc(1px + 50%)}@keyframes skeleton{0%{background-position:150%}to{background-position:-50%}}.table:where([dir=rtl],[dir=rtl] *){text-align:right}.table :where(th,td){padding:.75rem 1rem;vertical-align:middle}.table tr.active,.table tr.active:nth-child(2n),.table-zebra tbody tr:nth-child(2n){--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)))}.table-zebra tr.active,.table-zebra tr.active:nth-child(2n),.table-zebra-zebra tbody tr:nth-child(2n){--tw-bg-opacity:1;background-color:var(--fallback-b3,oklch(var(--b3)/var(--tw-bg-opacity)))}.table :where(thead tr,tbody tr:not(:last-child),tbody tr:first-child:last-child){border-bottom-width:1px;--tw-border-opacity:1;border-bottom-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-border-opacity)))}.table :where(thead,tfoot){white-space:nowrap;font-size:.75rem;line-height:1rem;font-weight:700;color:var(--fallback-bc,oklch(var(--bc)/.6))}.table :where(tfoot){border-top-width:1px;--tw-border-opacity:1;border-top-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-border-opacity)))}.textarea:focus{box-shadow:none;border-color:var(--fallback-bc,oklch(var(--bc)/.2));outline-style:solid;outline-width:2px;outline-offset:2px;outline-color:var(--fallback-bc,oklch(var(--bc)/.2))}.textarea-disabled,.textarea:disabled,.textarea[disabled]{cursor:not-allowed;--tw-border-opacity:1;border-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-border-opacity)));--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)));color:var(--fallback-bc,oklch(var(--bc)/.4))}.textarea-disabled::-moz-placeholder,.textarea:disabled::-moz-placeholder,.textarea[disabled]::-moz-placeholder{color:var(--fallback-bc,oklch(var(--bc)/var(--tw-placeholder-opacity)));--tw-placeholder-opacity:0.2}.textarea-disabled::placeholder,.textarea:disabled::placeholder,.textarea[disabled]::placeholder{color:var(--fallback-bc,oklch(var(--bc)/var(--tw-placeholder-opacity)));--tw-placeholder-opacity:0.2}@keyframes toast-pop{0%{transform:scale(.9);opacity:0}to{transform:scale(1);opacity:1}}.btm-nav-xs>:where(.active){border-top-width:1px}.btm-nav-sm>:where(.active){border-top-width:2px}.btm-nav-md>:where(.active){border-top-width:2px}.btm-nav-lg>:where(.active){border-top-width:4px}.btn-xs{height:1.5rem;min-height:1.5rem;padding-left:.5rem;padding-right:.5rem;font-size:.75rem}.btn-square:where(.btn-xs){height:1.5rem;width:1.5rem;padding:0}.btn-circle:where(.btn-xs){height:1.5rem;width:1.5rem;border-radius:9999px;padding:0}.join.join-vertical{flex-direction:column}.join.join-vertical .join-item:first-child:not(:last-child),.join.join-vertical :first-child:not(:last-child) .join-item{border-end-start-radius:0;border-end-end-radius:0;border-start-start-radius:inherit;border-start-end-radius:inherit}.join.join-vertical .join-item:last-child:not(:first-child),.join.join-vertical :last-child:not(:first-child) .join-item{border-start-start-radius:0;border-start-end-radius:0;border-end-start-radius:inherit;border-end-end-radius:inherit}.join.join-horizontal{flex-direction:row}.join.join-horizontal .join-item:first-child:not(:last-child),.join.join-horizontal :first-child:not(:last-child) .join-item{border-end-end-radius:0;border-start-end-radius:0;border-end-start-radius:inherit;border-start-start-radius:inherit}.join.join-horizontal .join-item:last-child:not(:first-child),.join.join-horizontal :last-child:not(:first-child) .join-item{border-end-start-radius:0;border-start-start-radius:0;border-end-end-radius:inherit;border-start-end-radius:inherit}.join.join-vertical>:where(:not(:first-child)){margin-left:0;margin-right:0;margin-top:-1px}.join.join-vertical>:where(:not(:first-child)):is(.btn){margin-top:calc(var(--border-btn)*-1)}.join.join-horizontal>:where(:not(:first-child)){margin-top:0;margin-bottom:0;margin-inline-start:-1px}.join.join-horizontal>:where(:not(:first-child)):is(.btn){margin-inline-start:calc(var(--border-btn)*-1)}.menu-sm :where(li:not(.menu-title)>:not(ul,details,.menu-title)),.menu-sm :where(li:not(.menu-title)>details>summary:not(.menu-title)){border-radius:var(--rounded-btn,.5rem);padding:.25rem .75rem;font-size:.875rem;line-height:1.25rem}.menu-sm .menu-title{padding:.5rem .75rem}.invisible{visibility:hidden}.static{position:static}.absolute{position:absolute}.relative{position:relative}.sticky{position:sticky}.left-0{left:0}.top-0{top:0}.top-4{top:1rem}.isolate{isolation:isolate}.m-0{margin:0}.mb-2{margin-bottom:.5rem}.ml-1{margin-left:.25rem}.ml-2{margin-left:.5rem}.mr-2{margin-right:.5rem}.mr-4{margin-right:1rem}.flex{display:flex}.table{display:table}.hidden{display:none}.size-5{width:1.25rem;height:1.25rem}.w-16{width:4rem}.w-56{width:14rem}.w-full{width:100%}.min-w-32{min-width:8rem}.max-w-32{max-width:8rem}.max-w-64{max-width:16rem}.max-w-full{max-width:100%}.items-center{align-items:center}.justify-center{justify-content:center}.justify-between{justify-content:space-between}.self-start{align-self:flex-start}.overflow-x-auto{overflow-x:auto}.truncate{overflow:hidden;text-overflow:ellipsis}.truncate,.whitespace-nowrap{white-space:nowrap}.rounded-box{border-radius:var(--rounded-box,1rem)}.border{border-width:1px}.border-neutral{--tw-border-opacity:1;border-color:var(--fallback-n,oklch(var(--n)/var(--tw-border-opacity)))}.bg-base-100{--tw-bg-opacity:1;background-color:var(--fallback-b1,oklch(var(--b1)/var(--tw-bg-opacity)))}.bg-base-200{--tw-bg-opacity:1;background-color:var(--fallback-b2,oklch(var(--b2)/var(--tw-bg-opacity)))}.px-1{padding-left:.25rem;padding-right:.25rem}.px-2{padding-left:.5rem;padding-right:.5rem}.py-0{padding-top:0;padding-bottom:0}.py-1{padding-top:.25rem;padding-bottom:.25rem}.pl-2{padding-left:.5rem}.text-left{text-align:left}.font-mono{font-family:ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,Liberation Mono,Courier New,monospace}.text-sm{font-size:.875rem;line-height:1.25rem}.text-xl{font-size:1.25rem;line-height:1.75rem}.text-xs{font-size:.75rem;line-height:1rem}.lowercase{text-transform:lowercase}.text-base-content{--tw-text-opacity:1;color:var(--fallback-bc,oklch(var(--bc)/var(--tw-text-opacity)))}.filter{filter:var(--tw-blur) var(--tw-brightness) var(--tw-contrast) var(--tw-grayscale) var(--tw-hue-rotate) var(--tw-invert) var(--tw-saturate) var(--tw-sepia) var(--tw-drop-shadow)}@media (min-width:640px){.sm\\:px-4{padding-left:1rem;padding-right:1rem}}` }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(
          "script",
          {
            defer: true,
            src: "https://cdn.jsdelivr.net/npm/alpinejs@3.14.0/dist/cdn.min.js"
          }
        ),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("script", { defer: true, src: "https://unpkg.com/htmx.org@1.9.12" })
      ] }),
      /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("body", { children: [
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("div", { class: "navbar bg-base-100", children: /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("a", { href: "/", class: "btn btn-ghost text-xl", children: "plainweb" }) }),
        /* @__PURE__ */ (0, import_jsx_runtime3.jsx)("div", { class: "px-2 sm:px-4", children: /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("div", { class: "flex", children: [
          /* @__PURE__ */ (0, import_jsx_runtime3.jsxs)("div", { class: "sticky top-4 self-start mr-4", children: [
            /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(MainNavigation, { active: props.active }),
            /* @__PURE__ */ (0, import_jsx_runtime3.jsx)(SubNavigation, { items: props.subNavigation, path: props.path })
          ] }),
          props.children
        ] }) })
      ] })
    ] })
  ] });
}

// src/admin/database/routes/[table]/index.tsx
var import_drizzle_orm8 = require("drizzle-orm");
var import_jsx_runtime4 = require("@kitajs/html/jsx-runtime");
var GET2 = async ({ req, res }) => {
  const tableName = req.params.table;
  const db = res.locals.database;
  const columns = db.all(
    import_drizzle_orm8.sql`SELECT * from pragma_table_info(${tableName}) LIMIT 100`
  );
  const rows = db.all(
    import_drizzle_orm8.sql`SELECT * FROM ${import_drizzle_orm8.sql.identifier(tableName)} LIMIT 100`
  );
  const tables = await db.select({ name: import_drizzle_orm8.sql`name` }).from(import_drizzle_orm8.sql`sqlite_master`).where(import_drizzle_orm8.sql`type=${"table"} AND name NOT LIKE ${"sqlite_%"}`);
  const tableCounts = [];
  for (const table of tables) {
    const [rowCount] = db.all(
      import_drizzle_orm8.sql`SELECT COUNT(*) as count FROM ${import_drizzle_orm8.sql.identifier(table.name)}`
    );
    tableCounts.push({ tableName: table.name, count: rowCount?.count ?? 0 });
  }
  const subNavigationItems = tableCounts.map(
    ({ tableName: tableName2, count }) => ({
      href: `${config.adminBasePath}/database/${tableName2}`,
      label: /* @__PURE__ */ (0, import_jsx_runtime4.jsxs)("span", { class: "flex justify-between items-center w-full", children: [
        /* @__PURE__ */ (0, import_jsx_runtime4.jsx)("span", { safe: true, children: tableName2 }),
        /* @__PURE__ */ (0, import_jsx_runtime4.jsx)("span", { class: "ml-2 text-xs text-base-content", children: count })
      ] })
    })
  );
  return /* @__PURE__ */ (0, import_jsx_runtime4.jsx)(
    AdminLayout,
    {
      subNavigation: subNavigationItems,
      active: "database",
      path: req.path,
      children: /* @__PURE__ */ (0, import_jsx_runtime4.jsx)("div", { class: "overflow-x-auto w-full whitespace-nowrap text-sm max-w-full", children: /* @__PURE__ */ (0, import_jsx_runtime4.jsxs)("table", { class: "font-mono", children: [
        /* @__PURE__ */ (0, import_jsx_runtime4.jsx)("thead", { children: /* @__PURE__ */ (0, import_jsx_runtime4.jsxs)("tr", { children: [
          /* @__PURE__ */ (0, import_jsx_runtime4.jsx)("th", {}),
          columns.map((column) => /* @__PURE__ */ (0, import_jsx_runtime4.jsx)(
            "th",
            {
              safe: true,
              class: "border border-neutral px-2 py-1 text-left max-w-64 min-w-32 truncate",
              children: column.name
            }
          ))
        ] }) }),
        /* @__PURE__ */ (0, import_jsx_runtime4.jsx)("tbody", { "hx-target": "closest tr", "hx-swap": "outerHTML", children: rows.map((row) => /* @__PURE__ */ (0, import_jsx_runtime4.jsx)(
          TableRow,
          {
            tableName,
            columns,
            row,
            editing: false
          }
        )) })
      ] }) })
    }
  );
};

// src/admin/database/routes/[table]/row.tsx
var import_drizzle_orm9 = require("drizzle-orm");
var import_jsx_runtime5 = require("@kitajs/html/jsx-runtime");
var GET3 = async ({ req, res }) => {
  const tableName = req.params.table;
  const db = res.locals.database;
  const row = JSON.parse(decodeURIComponent(req.query.row));
  const columns = db.all(
    import_drizzle_orm9.sql`SELECT * from pragma_table_info(${tableName}) LIMIT 100`
  );
  config.verbose > 1 && console.log("[admin] [database]", "fetching row", row);
  const whereClause = Object.entries(row).map(([column, value]) => import_drizzle_orm9.sql`${column} = ${value}`).join(" AND ");
  const query = import_drizzle_orm9.sql`SELECT * FROM ${import_drizzle_orm9.sql.identifier(tableName)} WHERE`;
  let i = 0;
  for (const [column, value] of Object.entries(row)) {
    query.append(import_drizzle_orm9.sql`${import_drizzle_orm9.sql.identifier(column)} = ${value}`);
    if (i === Object.keys(row).length - 1) continue;
    query.append(import_drizzle_orm9.sql` AND `);
    i++;
  }
  const found = db.all(query);
  if (found.length === 0) {
    throw new Error(`Row not found for ${whereClause}`);
  }
  return /* @__PURE__ */ (0, import_jsx_runtime5.jsx)(TableRow, { tableName, columns, row });
};

// src/admin/database/routes/index.tsx
var import_drizzle_orm10 = require("drizzle-orm");
var import_jsx_runtime6 = require("@kitajs/html/jsx-runtime");
var GET4 = async ({ req, res }) => {
  const db = res.locals.database;
  const tables = await db.select({ name: import_drizzle_orm10.sql`name` }).from(import_drizzle_orm10.sql`sqlite_master`).where(import_drizzle_orm10.sql`type=${"table"} AND name NOT LIKE ${"sqlite_%"}`);
  if (!tables[0]) {
    return /* @__PURE__ */ (0, import_jsx_runtime6.jsx)("div", { children: "No tables found" });
  }
  return redirect(`${config.adminBasePath}/database/${tables[0].name}`);
};

// src/admin/database/routes/sql.tsx
var GET5 = async ({ req, res }) => {
  throw new Error("Not implemented");
};
var POST2 = async ({ req, res }) => {
  throw new Error("Not implemented");
};

// src/admin/database/loaded-file-routes.ts
var databaseRoutes = [
  { filePath: "/[table]/index.tsx", GET: GET2 },
  { filePath: "/[table]/edit.tsx", GET, POST },
  { filePath: "/[table]/row.tsx", GET: GET3 },
  { filePath: "/index.tsx", GET: GET4 },
  { filePath: "/sql.tsx", GET: GET5, POST: POST2 }
];

// src/admin/admin.ts
var log8 = getLogger("admin");
async function adminRouter(opts) {
  const { database: database3, path: path5, verbose = 3 } = opts;
  config.adminBasePath = path5;
  const router = import_express4.default.Router();
  router.use((req, res, next) => {
    res.locals.database = database3;
    next();
  });
  router.use(
    "/database",
    await fileRouter({ dir: "", loadedFileRoutes: databaseRoutes })
  );
  router.use("/", (req, res) => {
    log8.info("redirecting / to /database");
    res.redirect(`${path5}/database`);
  });
  return router;
}

// src/plainstack.ts
var unstable_admin = adminRouter;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  defineConfig,
  defineHandler,
  defineInmemoryTask,
  defineJob,
  defineMiddleware,
  defineTask,
  fileRouter,
  getApp,
  getDatabase,
  getLogger,
  getWorker,
  html,
  isolate,
  json,
  log,
  middleware,
  migrate,
  notFound,
  outbox,
  perform,
  printRoutes,
  randomId,
  redirect,
  sendMail,
  stream,
  testHandler,
  unstable_admin
});
//# sourceMappingURL=plainstack.js.map